@@ -87,9 +87,53 @@ class ProcessPoolMixin(ExecutorMixin):
             )
 
 
+@unittest.skipIf(not SHARED_POOL, 'not-shared-pool')
+class ASharedPoolInitTest(unittest.TestCase):
+    executor_type = futures.MPIPoolExecutor
+
+    @unittest.skipIf(WORLD_SIZE == 1, 'world-size-1')
+    def test_initializer_1(self):
+        executor = self.executor_type(
+            initializer=sleep_and_raise,
+            initargs=(0.2,),
+        )
+        executor.submit(time.sleep, 0).cancel()
+        future = executor.submit(time.sleep, 0)
+        with self.assertRaises(futures.BrokenExecutor):
+            executor.submit(time.sleep, 0).result()
+        with self.assertRaises(futures.BrokenExecutor):
+            future.result()
+        with self.assertRaises(futures.BrokenExecutor):
+            executor.submit(time.sleep, 0)
+
+    @unittest.skipIf(WORLD_SIZE == 1, 'world-size-1')
+    def test_initializer_2(self):
+        executor = self.executor_type(
+            initializer=time.sleep,
+            initargs=(0,),
+        )
+        executor.bootup()
+        with self.assertRaises(futures.BrokenExecutor):
+            executor.submit(time.sleep, 0).result()
+        with self.assertRaises(futures.BrokenExecutor):
+            executor.submit(time.sleep, 0)
+
+    @unittest.skipIf(WORLD_SIZE == 1, 'world-size-1')
+    def test_initializer_3(self):
+        executor = self.executor_type()
+        executor.submit(time.sleep, 0).result()
+        executor.shutdown()
+
+
 class ProcessPoolInitTest(ProcessPoolMixin,
                           unittest.TestCase):
 
+    def setUp(self):
+        pass
+
+    def tearDown(self):
+        pass
+
     def _prime_executor(self):
         pass
 
@@ -139,16 +183,16 @@ class ProcessPoolInitTest(ProcessPoolMixin,
         self.assertTrue(future.result(), run_name)
 
     def test_max_workers_environ(self):
-        save = os.environ.get('MPI4PY_MAX_WORKERS')
-        os.environ['MPI4PY_MAX_WORKERS'] = '1'
+        save = os.environ.get('MPI4PY_FUTURES_MAX_WORKERS')
+        os.environ['MPI4PY_FUTURES_MAX_WORKERS'] = '1'
         try:
             executor = self.executor_type()
             executor.submit(time.sleep, 0).result()
             executor.shutdown()
         finally:
-            del os.environ['MPI4PY_MAX_WORKERS']
+            del os.environ['MPI4PY_FUTURES_MAX_WORKERS']
             if save is not None:
-                os.environ['MPI4PY_MAX_WORKERS'] = save
+                os.environ['MPI4PY_FUTURES_MAX_WORKERS'] = save
 
     def test_max_workers_negative(self):
         for number in (0, -1):
@@ -156,6 +200,43 @@ class ProcessPoolInitTest(ProcessPoolMixin,
                               self.executor_type,
                               max_workers=number)
 
+    @unittest.skipIf(SHARED_POOL, 'shared-pool')
+    def test_initializer(self):
+        executor = self.executor_type(
+            initializer=time.sleep,
+            initargs=(0,),
+        )
+        executor.submit(time.sleep, 0).result()
+
+    @unittest.skipIf(SHARED_POOL, 'shared-pool')
+    def test_initializer_bad(self):
+        with self.assertRaises(TypeError):
+            self.executor_type(initializer=123)
+
+    @unittest.skipIf(SHARED_POOL, 'shared-pool')
+    def test_initializer_error(self):
+        executor = self.executor_type(
+            initializer=sleep_and_raise,
+            initargs=(0.2,),
+        )
+        executor.submit(time.sleep, 0).cancel()
+        future = executor.submit(time.sleep, 0)
+        with self.assertRaises(futures.BrokenExecutor):
+            executor.submit(time.sleep, 0).result()
+        with self.assertRaises(futures.BrokenExecutor):
+            future.result()
+        with self.assertRaises(futures.BrokenExecutor):
+            executor.submit(time.sleep, 0)
+
+    @unittest.skipIf(SHARED_POOL, 'shared-pool')
+    def test_initializer_error_del(self):
+        executor = self.executor_type(
+            initializer=sleep_and_raise,
+            initargs=(0.2,),
+        )
+        executor.bootup()
+        del executor
+
 
 class ProcessPoolBootupTest(ProcessPoolMixin,
                             unittest.TestCase):
@@ -239,6 +320,29 @@ class ProcessPoolShutdownTest(ProcessPoolMixin,
         self.assertEqual(executor._pool, None)
         self.assertEqual(executor._shutdown, True)
 
+    def test_submit_shutdown_cancel(self):
+        executor = self.executor_type(max_workers=1)
+        executor.bootup()
+        num_workers = executor._pool.size
+        for _ in range(num_workers*10):
+            executor.submit(time.sleep, 0.1)
+        fut = executor.submit(time.sleep, 0)
+        executor.shutdown(wait=False, cancel_futures=False)
+        self.assertFalse(fut.cancelled())
+        executor.shutdown(wait=True, cancel_futures=True)
+        self.assertTrue(fut.cancelled())
+
+    def test_shutdown_cancel(self):
+        executor = self.executor_type(max_workers=1)
+        executor.bootup()
+        executor._pool.cancel()
+        executor.shutdown(wait=False, cancel_futures=False)
+        executor.shutdown(wait=False, cancel_futures=False)
+        executor.shutdown(wait=False, cancel_futures=True)
+        executor.shutdown(wait=False, cancel_futures=True)
+        executor.shutdown(wait=True,  cancel_futures=True)
+        executor.shutdown(wait=True,  cancel_futures=True)
+
     def test_init_bootup_shutdown(self):
         executor = self.executor_type(max_workers=1)
         self.assertEqual(executor._pool, None)
@@ -410,7 +514,7 @@ class AsCompletedTestMixin:
                 completed)
 
     def test_zero_timeout(self):
-        future1 = self.executor.submit(time.sleep, 0.2)
+        future1 = self.executor.submit(time.sleep, 0.5)
         completed_futures = set()
         try:
             for future in futures.as_completed(
@@ -430,7 +534,7 @@ class AsCompletedTestMixin:
 
     def test_nonzero_timeout(self):
         future1 = self.executor.submit(time.sleep, 0.0)
-        future2 = self.executor.submit(time.sleep, 0.2)
+        future2 = self.executor.submit(time.sleep, 0.5)
         completed_futures = set()
         try:
             for future in futures.as_completed(
@@ -438,7 +542,7 @@ class AsCompletedTestMixin:
                      EXCEPTION_FUTURE,
                      SUCCESSFUL_FUTURE,
                      future1],
-                    timeout=0.1):
+                    timeout=0.2):
                 completed_futures.add(future)
         except futures.TimeoutError:
             pass
@@ -508,9 +612,7 @@ class ExecutorTestMixin:
     def test_map_timeout(self):
         results = []
         try:
-            for i in self.executor.map(time.sleep,
-                                       [0, 0, 1],
-                                       timeout=0.25):
+            for i in self.executor.map(time.sleep, [0, 0, 1], timeout=0.25):
                 results.append(i)
         except futures.TimeoutError:
             pass
@@ -583,8 +685,8 @@ class ProcessPoolExecutorTest(ProcessPoolMixin,
         num_workers = self.executor._pool.size
         results = []
         try:
-            args = [0.2] + [0]*(num_workers-1)
-            for i in map_unordered(time.sleep, args, timeout=0.1):
+            args = [1] + [0]*(num_workers-1)
+            for i in map_unordered(time.sleep, args, timeout=0.25):
                 results.append(i)
         except futures.TimeoutError:
             pass
@@ -667,31 +769,6 @@ class ProcessPoolSubmitTest(unittest.TestCase):
         finally:
             serialized.lock = lock_save
 
-    def orig_test_mpi_serialized_support(self):
-        threading = futures._lib.threading
-        serialized = futures._lib.serialized
-        lock_save = serialized.lock
-        try:
-            serialized.lock = threading.Lock()
-            executor = futures.MPIPoolExecutor(1).bootup()
-            executor.submit(abs, 0).result()
-            if lock_save is not None:
-                serialized.lock = None
-                with lock_save:
-                    executor.submit(abs, 0).result()
-            serialized.lock = lock_save
-            executor.submit(abs, 0).result()
-            executor.shutdown()
-            if lock_save is not None:
-                serialized.lock = None
-                with lock_save:
-                    executor = futures.MPIPoolExecutor(1).bootup()
-                    executor.submit(abs, 0).result()
-                    executor.shutdown()
-                serialized.lock = lock_save
-        finally:
-            serialized.lock = lock_save
-
     def test_shared_executors(self):
         if not SHARED_POOL: return
         executors = [futures.MPIPoolExecutor() for _ in range(16)]
@@ -828,7 +905,7 @@ class MPICommExecutorTest(unittest.TestCase):
 
             list(executor.map(time.sleep, [0, 0]))
             list(executor.map(time.sleep, [0, 0], timeout=1))
-            iterator = executor.map(time.sleep, [0.1, 0], timeout=0)
+            iterator = executor.map(time.sleep, [0.2, 0], timeout=0)
             self.assertRaises(futures.TimeoutError, list, iterator)
 
     def test_args(self):
@@ -863,8 +940,7 @@ class MPICommExecutorTest(unittest.TestCase):
 
     @unittest.skipIf(SHARED_POOL, 'shared-pool')
     def test_arg_comm_bad(self):
-        if MPI.COMM_WORLD.Get_size() == 1:
-            return
+        if MPI.COMM_WORLD.Get_size() == 1: return
         intercomm = futures._lib.comm_split(MPI.COMM_WORLD)
         try:
             self.assertRaises(ValueError, self.MPICommExecutor, intercomm)
@@ -882,6 +958,39 @@ class MPICommExecutorTest(unittest.TestCase):
             else:
                 self.fail('expected RuntimeError')
 
+    @unittest.skipIf(SHARED_POOL, 'shared-pool')
+    def test_initializer(self):
+        mpicommexecutor = self.MPICommExecutor(
+            initializer=time.sleep,
+            initargs=(0,),
+        )
+        with mpicommexecutor as executor:
+            if executor is not None:
+                executor.bootup()
+                del executor
+        with mpicommexecutor as executor:
+            if executor is not None:
+                executor.submit(time.sleep, 0).result()
+
+    @unittest.skipIf(SHARED_POOL, 'shared-pool')
+    def test_initializer_error(self):
+        mpicommexecutor = self.MPICommExecutor(
+            initializer=sleep_and_raise,
+            initargs=(0.2,),
+        )
+        with mpicommexecutor as executor:
+            if executor is not None:
+                executor.bootup()
+                del executor
+        with mpicommexecutor as executor:
+            if executor is not None:
+                executor.submit(time.sleep, 0).cancel()
+                future = executor.submit(time.sleep, 0)
+                with self.assertRaises(futures.BrokenExecutor):
+                    executor.submit(time.sleep, 0).result()
+                with self.assertRaises(futures.BrokenExecutor):
+                    future.result()
+
 
 from mpi4py.futures.aplus import ThenableFuture
 
@@ -1159,7 +1268,9 @@ class ThenTest(unittest.TestCase):
         def transform(a):
             try:
                 f = chain.pop(0)
-                f.set_result(transform(a))
+                r = transform(a)
+                f.__init__()
+                f.set_result(r)
                 return f
             except IndexError:
                 return 42
@@ -1178,7 +1289,8 @@ class ThenTest(unittest.TestCase):
         self.assert_(new_f.exception())
         with self.assertRaises(RuntimeError) as catcher:
             new_f.result()
-        assert 'Circular future chain detected' in catcher.exception.args[0]
+        self.assert_('Circular future chain detected'
+                     in catcher.exception.args[0])
 
 
 SKIP_POOL_TEST = False
@@ -1188,9 +1300,24 @@ if name == 'Open MPI':
         SKIP_POOL_TEST = True
     if version == (4,0,0):
         SKIP_POOL_TEST = True
+    if version == (4,0,1) and sys.platform=='darwin':
+        SKIP_POOL_TEST = True
+    if version == (4,0,2) and sys.platform=='darwin':
+        SKIP_POOL_TEST = True
 if name == 'MPICH':
+    if sys.platform == 'darwin':
+        if version >= (3, 4) and version < (4, 0):
+            SKIP_POOL_TEST = True
     if MPI.COMM_WORLD.Get_attr(MPI.APPNUM) is None:
         SKIP_POOL_TEST = True
+    try:
+        port = MPI.Open_port()
+        MPI.Close_port(port)
+    except:
+        port = ""
+    if port == "":
+        SKIP_POOL_TEST = True
+    del port
 if name == 'MVAPICH2':
     SKIP_POOL_TEST = True
 if name == 'MPICH2':
@@ -1210,8 +1337,15 @@ if MPI.Get_version() < (2,0):
 if SHARED_POOL:
     del MPICommExecutorTest.test_arg_root
     del MPICommExecutorTest.test_arg_comm_bad
+    del MPICommExecutorTest.test_initializer
+    del MPICommExecutorTest.test_initializer_error
     del ProcessPoolInitTest.test_init_globals
+    del ProcessPoolInitTest.test_initializer
+    del ProcessPoolInitTest.test_initializer_bad
+    del ProcessPoolInitTest.test_initializer_error
+    del ProcessPoolInitTest.test_initializer_error_del
     if WORLD_SIZE == 1:
+        del ASharedPoolInitTest
         del ProcessPoolInitTest.test_run_name
         del ProcessPoolPickleTest
 elif WORLD_SIZE > 1 or SKIP_POOL_TEST:
@@ -1223,6 +1357,8 @@ elif WORLD_SIZE > 1 or SKIP_POOL_TEST:
     del ProcessPoolExecutorTest
     del ProcessPoolSubmitTest
     del ProcessPoolPickleTest
+if not SHARED_POOL:
+    del ASharedPoolInitTest
 
 
 if __name__ == '__main__':
