@@ -49,6 +49,18 @@ def getoptionparser():
     parser.add_option("--vt",
                       action="store_true", dest="vt", default=False,
                       help="use VampirTrace for MPI profiling")
+    parser.add_option("--cupy",
+                      action="store_true", dest="cupy", default=False,
+                      help="enable testing with CuPy arrays")
+    parser.add_option("--no-cupy",
+                      action="store_false", dest="cupy", default=False,
+                      help="disable testing with CuPy arrays")
+    parser.add_option("--numba",
+                      action="store_true", dest="numba", default=False,
+                      help="enable testing with Numba arrays")
+    parser.add_option("--no-numba",
+                      action="store_false", dest="numba", default=False,
+                      help="disable testing with Numba arrays")
     parser.add_option("--no-numpy",
                       action="store_false", dest="numpy", default=True,
                       help="disable testing with NumPy arrays")
@@ -62,7 +74,8 @@ def getoptionparser():
 
 def getbuilddir():
     from distutils.util import get_platform
-    s = os.path.join("build", "lib.%s-%.3s" % (get_platform(), sys.version))
+    plat_name, (x, y) = get_platform(), sys.version_info[:2]
+    s = os.path.join("build", "lib.%s-%d.%d" % (plat_name, x, y))
     if hasattr(sys, 'gettotalrefcount'): s += '-pydebug'
     return s
 
@@ -95,6 +108,10 @@ def setup_unittest(options):
 
 def import_package(options, pkgname):
     #
+    if not options.cupy:
+        sys.modules['cupy'] = None
+    if not options.numba:
+        sys.modules['numba'] = None
     if not options.numpy:
         sys.modules['numpy'] = None
     if not options.array:
@@ -174,6 +191,10 @@ def load_tests(options, args):
         testnames.append(testname)
     testnames.sort()
     # Handle options
+    if not options.cupy:
+        sys.modules['cupy'] = None
+    if not options.numba:
+        sys.modules['numba'] = None
     if not options.numpy:
         sys.modules['numpy'] = None
     if not options.array:
